/*
 * $Id$
 * Copyright (C) 2001-2002 The Apache Software Foundation. All rights reserved.
 * For details on use and redistribution please refer to the
 * LICENSE file included with these sources.
 * @author <a href="mailto:pbwest@powerup.com.au">Peter B. West</a>
 * @version $Revision$ $Name$
 */

package org.apache.fop.fo;

import org.apache.fop.fo.expr.PropertyException;

/**
 * A class of constants; an array of all property names and the constants
 * by which to refer to them.
 */

public class PropNames {

    /*
     * The list of property constants can be regenerated in XEmacs by setting
     * the region on the list of constants. (C-Space at the beginning,
     * move to last line, C-x C-x to exchange mark and point.)  Then run
     * a shell command on the region with replacement (M-1 M-|).  Use
     * the perl command:
     * perl -p -e 'BEGIN{$n=0};$n++ if s/= [0-9]+,/= $n,/'
     * Alternatively, start at a given point in the list by setting the
     * appropriate start value for $n.
     *
     * in vi, set mark `a' at the last line and
     * !'aperl... etc
     */
     /** Constant for matching property defined in <i>XSLFO</i>. */

    public static final int
                                    NO_PROPERTY = 0,
                              ABSOLUTE_POSITION = 1,
                                   ACTIVE_STATE = 2,
                               ALIGNMENT_ADJUST = 3,
                             ALIGNMENT_BASELINE = 4,
                                   AUTO_RESTORE = 5,
                                        AZIMUTH = 6,
                                     BACKGROUND = 7,
                          BACKGROUND_ATTACHMENT = 8,
                               BACKGROUND_COLOR = 9,
                               BACKGROUND_IMAGE = 10,
                            BACKGROUND_POSITION = 11,
                 BACKGROUND_POSITION_HORIZONTAL = 12,
                   BACKGROUND_POSITION_VERTICAL = 13,
                              BACKGROUND_REPEAT = 14,
                                 BASELINE_SHIFT = 15,
                             BLANK_OR_NOT_BLANK = 16,
                    BLOCK_PROGRESSION_DIMENSION = 17,
            BLOCK_PROGRESSION_DIMENSION_MINIMUM = 18,
            BLOCK_PROGRESSION_DIMENSION_OPTIMUM = 19,
            BLOCK_PROGRESSION_DIMENSION_MAXIMUM = 20,
                                         BORDER = 21,
                             BORDER_AFTER_COLOR = 22,
                        BORDER_AFTER_PRECEDENCE = 23,
                             BORDER_AFTER_STYLE = 24,
                             BORDER_AFTER_WIDTH = 25,
                      BORDER_AFTER_WIDTH_LENGTH = 26,
              BORDER_AFTER_WIDTH_CONDITIONALITY = 27,
                            BORDER_BEFORE_COLOR = 28,
                       BORDER_BEFORE_PRECEDENCE = 29,
                            BORDER_BEFORE_STYLE = 30,
                            BORDER_BEFORE_WIDTH = 31,
                     BORDER_BEFORE_WIDTH_LENGTH = 32,
             BORDER_BEFORE_WIDTH_CONDITIONALITY = 33,
                                  BORDER_BOTTOM = 34,
                            BORDER_BOTTOM_COLOR = 35,
                            BORDER_BOTTOM_STYLE = 36,
                            BORDER_BOTTOM_WIDTH = 37,
                                BORDER_COLLAPSE = 38,
                                   BORDER_COLOR = 39,
                               BORDER_END_COLOR = 40,
                          BORDER_END_PRECEDENCE = 41,
                               BORDER_END_STYLE = 42,
                               BORDER_END_WIDTH = 43,
                        BORDER_END_WIDTH_LENGTH = 44,
                BORDER_END_WIDTH_CONDITIONALITY = 45,
                                    BORDER_LEFT = 46,
                              BORDER_LEFT_COLOR = 47,
                              BORDER_LEFT_STYLE = 48,
                              BORDER_LEFT_WIDTH = 49,
                                   BORDER_RIGHT = 50,
                             BORDER_RIGHT_COLOR = 51,
                             BORDER_RIGHT_STYLE = 52,
                             BORDER_RIGHT_WIDTH = 53,
                              BORDER_SEPARATION = 54,
  BORDER_SEPARATION_BLOCK_PROGRESSION_DIRECTION = 55,
 BORDER_SEPARATION_INLINE_PROGRESSION_DIRECTION = 56,
                                 BORDER_SPACING = 57,
                             BORDER_START_COLOR = 58,
                        BORDER_START_PRECEDENCE = 59,
                             BORDER_START_STYLE = 60,
                             BORDER_START_WIDTH = 61,
                      BORDER_START_WIDTH_LENGTH = 62,
              BORDER_START_WIDTH_CONDITIONALITY = 63,
                                   BORDER_STYLE = 64,
                                     BORDER_TOP = 65,
                               BORDER_TOP_COLOR = 66,
                               BORDER_TOP_STYLE = 67,
                               BORDER_TOP_WIDTH = 68,
                                   BORDER_WIDTH = 69,
                                         BOTTOM = 70,
                                    BREAK_AFTER = 71,
                                   BREAK_BEFORE = 72,
                                   CAPTION_SIDE = 73,
                                      CASE_NAME = 74,
                                     CASE_TITLE = 75,
                                      CHARACTER = 76,
                                          CLEAR = 77,
                                           CLIP = 78,
                                          COLOR = 79,
                             COLOR_PROFILE_NAME = 80,
                                   COLUMN_COUNT = 81,
                                     COLUMN_GAP = 82,
                                  COLUMN_NUMBER = 83,
                                   COLUMN_WIDTH = 84,
                                 CONTENT_HEIGHT = 85,
                                   CONTENT_TYPE = 86,
                                  CONTENT_WIDTH = 87,
                                        COUNTRY = 88,
                                            CUE = 89,
                                      CUE_AFTER = 90,
                                     CUE_BEFORE = 91,
                   DESTINATION_PLACEMENT_OFFSET = 92,
                                      DIRECTION = 93,
                                  DISPLAY_ALIGN = 94,
                              DOMINANT_BASELINE = 95,
                                      ELEVATION = 96,
                                    EMPTY_CELLS = 97,
                                     END_INDENT = 98,
                                       ENDS_ROW = 99,
                                         EXTENT = 100,
                           EXTERNAL_DESTINATION = 101,
                                          FLOAT = 102,
                                      FLOW_NAME = 103,
                                           FONT = 104,
                                    FONT_FAMILY = 105,
                        FONT_SELECTION_STRATEGY = 106,
                                      FONT_SIZE = 107,
                               FONT_SIZE_ADJUST = 108,
                                   FONT_STRETCH = 109,
                                     FONT_STYLE = 110,
                                   FONT_VARIANT = 111,
                                    FONT_WEIGHT = 112,
                               FORCE_PAGE_COUNT = 113,
                                         FORMAT = 114,
                   GLYPH_ORIENTATION_HORIZONTAL = 115,
                     GLYPH_ORIENTATION_VERTICAL = 116,
                             GROUPING_SEPARATOR = 117,
                                  GROUPING_SIZE = 118,
                                         HEIGHT = 119,
                                      HYPHENATE = 120,
                          HYPHENATION_CHARACTER = 121,
                               HYPHENATION_KEEP = 122,
                       HYPHENATION_LADDER_COUNT = 123,
               HYPHENATION_PUSH_CHARACTER_COUNT = 124,
             HYPHENATION_REMAIN_CHARACTER_COUNT = 125,
                                             ID = 126,
                           INDICATE_DESTINATION = 127,
                            INITIAL_PAGE_NUMBER = 128,
                   INLINE_PROGRESSION_DIMENSION = 129,
           INLINE_PROGRESSION_DIMENSION_MINIMUM = 130,
           INLINE_PROGRESSION_DIMENSION_OPTIMUM = 131,
           INLINE_PROGRESSION_DIMENSION_MAXIMUM = 132,
                           INTERNAL_DESTINATION = 133,
                             INTRUSION_DISPLACE = 134,
                                  KEEP_TOGETHER = 135,
                      KEEP_TOGETHER_WITHIN_LINE = 136,
                      KEEP_TOGETHER_WITHIN_PAGE = 137,
                    KEEP_TOGETHER_WITHIN_COLUMN = 138,
                                 KEEP_WITH_NEXT = 139,
                     KEEP_WITH_NEXT_WITHIN_LINE = 140,
                     KEEP_WITH_NEXT_WITHIN_PAGE = 141,
                   KEEP_WITH_NEXT_WITHIN_COLUMN = 142,
                             KEEP_WITH_PREVIOUS = 143,
                 KEEP_WITH_PREVIOUS_WITHIN_LINE = 144,
                 KEEP_WITH_PREVIOUS_WITHIN_PAGE = 145,
               KEEP_WITH_PREVIOUS_WITHIN_COLUMN = 146,
                                       LANGUAGE = 147,
                           LAST_LINE_END_INDENT = 148,
                               LEADER_ALIGNMENT = 149,
                                  LEADER_LENGTH = 150,
                          LEADER_LENGTH_MINIMUM = 151,
                          LEADER_LENGTH_OPTIMUM = 152,
                          LEADER_LENGTH_MAXIMUM = 153,
                                 LEADER_PATTERN = 154,
                           LEADER_PATTERN_WIDTH = 155,
                                           LEFT = 156,
                                 LETTER_SPACING = 157,
                                   LETTER_VALUE = 158,
                             LINEFEED_TREATMENT = 159,
                                    LINE_HEIGHT = 160,
                            LINE_HEIGHT_MINIMUM = 161,
                            LINE_HEIGHT_OPTIMUM = 162,
                            LINE_HEIGHT_MAXIMUM = 163,
                     LINE_HEIGHT_CONDITIONALITY = 164,
                         LINE_HEIGHT_PRECEDENCE = 165,
                   LINE_HEIGHT_SHIFT_ADJUSTMENT = 166,
                         LINE_STACKING_STRATEGY = 167,
                                         MARGIN = 168,
                                  MARGIN_BOTTOM = 169,
                                    MARGIN_LEFT = 170,
                                   MARGIN_RIGHT = 171,
                                     MARGIN_TOP = 172,
                              MARKER_CLASS_NAME = 173,
                                    MASTER_NAME = 174,
                               MASTER_REFERENCE = 175,
                                     MAX_HEIGHT = 176,
                                MAXIMUM_REPEATS = 177,
                                      MAX_WIDTH = 178,
                                    MEDIA_USAGE = 179,
                                     MIN_HEIGHT = 180,
                                      MIN_WIDTH = 181,
                        NUMBER_COLUMNS_REPEATED = 182,
                         NUMBER_COLUMNS_SPANNED = 183,
                            NUMBER_ROWS_SPANNED = 184,
                                    ODD_OR_EVEN = 185,
                                        ORPHANS = 186,
                                       OVERFLOW = 187,
                                        PADDING = 188,
                                  PADDING_AFTER = 189,
                           PADDING_AFTER_LENGTH = 190,
                   PADDING_AFTER_CONDITIONALITY = 191,
                                 PADDING_BEFORE = 192,
                          PADDING_BEFORE_LENGTH = 193,
                  PADDING_BEFORE_CONDITIONALITY = 194,
                                 PADDING_BOTTOM = 195,
                                    PADDING_END = 196,
                             PADDING_END_LENGTH = 197,
                     PADDING_END_CONDITIONALITY = 198,
                                   PADDING_LEFT = 199,
                                  PADDING_RIGHT = 200,
                                  PADDING_START = 201,
                           PADDING_START_LENGTH = 202,
                   PADDING_START_CONDITIONALITY = 203,
                                    PADDING_TOP = 204,
                               PAGE_BREAK_AFTER = 205,
                              PAGE_BREAK_BEFORE = 206,
                              PAGE_BREAK_INSIDE = 207,
                                    PAGE_HEIGHT = 208,
                                  PAGE_POSITION = 209,
                                     PAGE_WIDTH = 210,
                                          PAUSE = 211,
                                    PAUSE_AFTER = 212,
                                   PAUSE_BEFORE = 213,
                                          PITCH = 214,
                                    PITCH_RANGE = 215,
                                    PLAY_DURING = 216,
                                       POSITION = 217,
                                     PRECEDENCE = 218,
            PROVISIONAL_DISTANCE_BETWEEN_STARTS = 219,
                   PROVISIONAL_LABEL_SEPARATION = 220,
                          REFERENCE_ORIENTATION = 221,
                                         REF_ID = 222,
                                    REGION_NAME = 223,
                                 RELATIVE_ALIGN = 224,
                              RELATIVE_POSITION = 225,
                               RENDERING_INTENT = 226,
                              RETRIEVE_BOUNDARY = 227,
                            RETRIEVE_CLASS_NAME = 228,
                              RETRIEVE_POSITION = 229,
                                       RICHNESS = 230,
                                          RIGHT = 231,
                                           ROLE = 232,
                                     RULE_STYLE = 233,
                                 RULE_THICKNESS = 234,
                                        SCALING = 235,
                                 SCALING_METHOD = 236,
                                   SCORE_SPACES = 237,
                                         SCRIPT = 238,
                               SHOW_DESTINATION = 239,
                                           SIZE = 240,
                                SOURCE_DOCUMENT = 241,
                                    SPACE_AFTER = 242,
                            SPACE_AFTER_MINIMUM = 243,
                            SPACE_AFTER_OPTIMUM = 244,
                            SPACE_AFTER_MAXIMUM = 245,
                     SPACE_AFTER_CONDITIONALITY = 246,
                         SPACE_AFTER_PRECEDENCE = 247,
                                   SPACE_BEFORE = 248,
                           SPACE_BEFORE_MINIMUM = 249,
                           SPACE_BEFORE_OPTIMUM = 250,
                           SPACE_BEFORE_MAXIMUM = 251,
                    SPACE_BEFORE_CONDITIONALITY = 252,
                        SPACE_BEFORE_PRECEDENCE = 253,
                                      SPACE_END = 254,
                              SPACE_END_MINIMUM = 255,
                              SPACE_END_OPTIMUM = 256,
                              SPACE_END_MAXIMUM = 257,
                       SPACE_END_CONDITIONALITY = 258,
                           SPACE_END_PRECEDENCE = 259,
                                    SPACE_START = 260,
                            SPACE_START_MINIMUM = 261,
                            SPACE_START_OPTIMUM = 262,
                            SPACE_START_MAXIMUM = 263,
                     SPACE_START_CONDITIONALITY = 264,
                         SPACE_START_PRECEDENCE = 265,
                                           SPAN = 266,
                                          SPEAK = 267,
                                   SPEAK_HEADER = 268,
                                  SPEAK_NUMERAL = 269,
                              SPEAK_PUNCTUATION = 270,
                                    SPEECH_RATE = 271,
                                            SRC = 272,
                                   START_INDENT = 273,
                                 STARTING_STATE = 274,
                                     STARTS_ROW = 275,
                                         STRESS = 276,
                         SUPPRESS_AT_LINE_BREAK = 277,
                                      SWITCH_TO = 278,
                                   TABLE_LAYOUT = 279,
                     TABLE_OMIT_FOOTER_AT_BREAK = 280,
                     TABLE_OMIT_HEADER_AT_BREAK = 281,
                    TARGET_PRESENTATION_CONTEXT = 282,
                      TARGET_PROCESSING_CONTEXT = 283,
                              TARGET_STYLESHEET = 284,
                                     TEXT_ALIGN = 285,
                                TEXT_ALIGN_LAST = 286,
                                  TEXT_ALTITUDE = 287,
                                TEXT_DECORATION = 288,
                                     TEXT_DEPTH = 289,
                                    TEXT_INDENT = 290,
                                    TEXT_SHADOW = 291,
                                 TEXT_TRANSFORM = 292,
                                            TOP = 293,
                            TREAT_AS_WORD_SPACE = 294,
                                   UNICODE_BIDI = 295,
                                 VERTICAL_ALIGN = 296,
                                     VISIBILITY = 297,
                                   VOICE_FAMILY = 298,
                                         VOLUME = 299,
                                    WHITE_SPACE = 300,
                           WHITE_SPACE_COLLAPSE = 301,
                          WHITE_SPACE_TREATMENT = 302,
                                         WIDOWS = 303,
                                          WIDTH = 304,
                                   WORD_SPACING = 305,
                                    WRAP_OPTION = 306,
                                   WRITING_MODE = 307,
                                       XML_LANG = 308,
                                        Z_INDEX = 309,
        
                            LAST_PROPERTY_INDEX = Z_INDEX;


    /**
     * A String[] array containing the names of all of the FO properties.
     * The array is effectively 1-based, as the first element is null.
     * The list of int constants referring to the properties must be manually
     * kept in sync with the names in this array, as the constants can be
     * used to index into this, and the other property arrays.
     */

    private static final String[] propertyNames = {
        "no-property",
        "absolute-position",
        "active-state",
        "alignment-adjust",
        "alignment-baseline",
        "auto-restore",
        "azimuth",
        "background",
        "background-attachment",
        "background-color",
        "background-image",
        "background-position",
        "background-position-horizontal",
        "background-position-vertical",
        "background-repeat",
        "baseline-shift",
        "blank-or-not-blank",
        "block-progression-dimension",
        "block-progression-dimension.minimum",
        "block-progression-dimension.optimum",
        "block-progression-dimension.maximum",
        "border",
        "border-after-color",
        "border-after-precedence",
        "border-after-style",
        "border-after-width",
        "border-after-width.length",
        "border-after-width.conditionality",
        "border-before-color",
        "border-before-precedence",
        "border-before-style",
        "border-before-width",
        "border-before-width.length",
        "border-before-width.conditionality",
        "border-bottom",
        "border-bottom-color",
        "border-bottom-style",
        "border-bottom-width",
        "border-collapse",
        "border-color",
        "border-end-color",
        "border-end-precedence",
        "border-end-style",
        "border-end-width",
        "border-end-width.length",
        "border-end-width.conditionality",
        "border-left",
        "border-left-color",
        "border-left-style",
        "border-left-width",
        "border-right",
        "border-right-color",
        "border-right-style",
        "border-right-width",
        "border-separation",
        "border-separation.block-progression-direction",
        "border-separation.inline-progression-direction",
        "border-spacing",
        "border-start-color",
        "border-start-precedence",
        "border-start-style",
        "border-start-width",
        "border-start-width.length",
        "border-start-width.conditionality",
        "border-style",
        "border-top",
        "border-top-color",
        "border-top-style",
        "border-top-width",
        "border-width",
        "bottom",
        "break-after",
        "break-before",
        "caption-side",
        "case-name",
        "case-title",
        "character",
        "clear",
        "clip",
        "color",
        "color-profile-name",
        "column-count",
        "column-gap",
        "column-number",
        "column-width",
        "content-height",
        "content-type",
        "content-width",
        "country",
        "cue",
        "cue-after",
        "cue-before",
        "destination-placement-offset",
        "direction",
        "display-align",
        "dominant-baseline",
        "elevation",
        "empty-cells",
        "end-indent",
        "ends-row",
        "extent",
        "external-destination",
        "float",
        "flow-name",
        "font",
        "font-family",
        "font-selection-strategy",
        "font-size",
        "font-size-adjust",
        "font-stretch",
        "font-style",
        "font-variant",
        "font-weight",
        "force-page-count",
        "format",
        "glyph-orientation-horizontal",
        "glyph-orientation-vertical",
        "grouping-separator",
        "grouping-size",
        "height",
        "hyphenate",
        "hyphenation-character",
        "hyphenation-keep",
        "hyphenation-ladder-count",
        "hyphenation-push-character-count",
        "hyphenation-remain-character-count",
        "id",
        "indicate-destination",
        "initial-page-number",
        "inline-progression-dimension",
        "inline-progression-dimension.minimum",
        "inline-progression-dimension.optimum",
        "inline-progression-dimension.maximum",
        "internal-destination",
        "intrusion-displace",
        "keep-together",
        "keep-together.within-line",
        "keep-together.within-column",
        "keep-together.within-page",
        "keep-with-next",
        "keep-with-next.within-line",
        "keep-with-next.within-column",
        "keep-with-next.within-page",
        "keep-with-previous",
        "keep-with-previous.within-line",
        "keep-with-previous.within-column",
        "keep-with-previous.within-page",
        "language",
        "last-line-end-indent",
        "leader-alignment",
        "leader-length",
        "leader-length.minimum",
        "leader-length.optimum",
        "leader-length.maximum",
        "leader-pattern",
        "leader-pattern-width",
        "left",
        "letter-spacing",
        "letter-value",
        "linefeed-treatment",
        "line-height",
        "line-height.minimum",
        "line-height.optimum",
        "line-height.maximum",
        "line-height.conditionality",
        "line-height.precedence",
        "line-height-shift-adjustment",
        "line-stacking-strategy",
        "margin",
        "margin-bottom",
        "margin-left",
        "margin-right",
        "margin-top",
        "marker-class-name",
        "master-name",
        "master-reference",
        "max-height",
        "maximum-repeats",
        "max-width",
        "media-usage",
        "min-height",
        "min-width",
        "number-columns-repeated",
        "number-columns-spanned",
        "number-rows-spanned",
        "odd-or-even",
        "orphans",
        "overflow",
        "padding",
        "padding-after",
        "padding-after.length",
        "padding-after.conditionality",
        "padding-before",
        "padding-before.length",
        "padding-before.conditionality",
        "padding-bottom",
        "padding-end",
        "padding-end.length",
        "padding-end.conditionality",
        "padding-left",
        "padding-right",
        "padding-start",
        "padding-start.length",
        "padding-start.conditionality",
        "padding-top",
        "page-break-after",
        "page-break-before",
        "page-break-inside",
        "page-height",
        "page-position",
        "page-width",
        "pause",
        "pause-after",
        "pause-before",
        "pitch",
        "pitch-range",
        "play-during",
        "position",
        "precedence",
        "provisional-distance-between-starts",
        "provisional-label-separation",
        "reference-orientation",
        "ref-id",
        "region-name",
        "relative-align",
        "relative-position",
        "rendering-intent",
        "retrieve-boundary",
        "retrieve-class-name",
        "retrieve-position",
        "richness",
        "right",
        "role",
        "rule-style",
        "rule-thickness",
        "scaling",
        "scaling-method",
        "score-spaces",
        "script",
        "show-destination",
        "size",
        "source-document",
        "space-after",
        "space-after.minimum",
        "space-after.optimum",
        "space-after.maximum",
        "space-after.conditionality",
        "space-after.precedence",
        "space-before",
        "space-before.minimum",
        "space-before.optimum",
        "space-before.maximum",
        "space-before.conditionality",
        "space-before.precedence",
        "space-end",
        "space-end.minimum",
        "space-end.optimum",
        "space-end.maximum",
        "space-end.conditionality",
        "space-end.precedence",
        "space-start",
        "space-start.minimum",
        "space-start.optimum",
        "space-start.maximum",
        "space-start.conditionality",
        "space-start.precedence",
        "span",
        "speak",
        "speak-header",
        "speak-numeral",
        "speak-punctuation",
        "speech-rate",
        "src",
        "start-indent",
        "starting-state",
        "starts-row",
        "stress",
        "suppress-at-line-break",
        "switch-to",
        "table-layout",
        "table-omit-footer-at-break",
        "table-omit-header-at-break",
        "target-presentation-context",
        "target-processing-context",
        "target-stylesheet",
        "text-align",
        "text-align-last",
        "text-altitude",
        "text-decoration",
        "text-depth",
        "text-indent",
        "text-shadow",
        "text-transform",
        "top",
        "treat-as-word-space",
        "unicode-bidi",
        "vertical-align",
        "visibility",
        "voice-family",
        "volume",
        "white-space",
        "white-space-collapse",
        "white-space-treatment",
        "widows",
        "width",
        "word-spacing",
        "wrap-option",
        "writing-mode",
        "xml:lang",
        "z-index"
    };

    /**
     * @param propindex <tt>int</tt> index of the FO property.
     * @return <tt>String</tt> name of the indexd FO property.
     * @exception PropertyException if the property index is invalid.
     */
    public static String getPropertyName(int propindex)
                throws PropertyException
    {
        if (propindex < 0 || propindex > LAST_PROPERTY_INDEX)
                throw new PropertyException
                        ("getPropertyName: index is invalid: " + propindex);
        return propertyNames[propindex];
    }

}
